@@ -17,7 +17,7 @@ use Image::ExifTool::Exif;
 use Image::ExifTool::MakerNotes;
 use Image::ExifTool::CanonRaw;
 
-$VERSION = '1.20';
+$VERSION = '1.22';
 
 sub ProcessOriginalRaw($$$);
 sub ProcessAdobeData($$$);
@@ -172,7 +172,7 @@ sub ProcessOriginalRaw($$$)
             if (($$et{OPTIONS}{Binary} and not $$et{EXCL_TAG_LOOKUP}{$lcTag}) or
                 $$et{REQ_TAG_LOOKUP}{$lcTag})
             {
-                unless (eval 'require Compress::Zlib') {
+                unless (eval { require Compress::Zlib }) {
                     $err = 'Install Compress::Zlib to extract compressed images';
                     last;
                 }
@@ -420,7 +420,7 @@ sub ProcessAdobeCRW($$$)
                 } elsif ($$newTags{$tagID}) {
                     my $nvHash = $et->GetNewValueHash($tagInfo);
                     if ($et->IsOverwriting($nvHash, $val)) {
-                        my $newVal = $et->GetNewValues($nvHash);
+                        my $newVal = $et->GetNewValue($nvHash);
                         my $verboseVal;
                         $verboseVal = $newVal if $verbose > 1;
                         # convert to specified format if necessary
@@ -813,7 +813,7 @@ information in DNG (Digital Negative) images.
 
 =head1 AUTHOR
 
-Copyright 2003-2014, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2018, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
