@@ -1,9 +1,10 @@
 #------------------------------------------------------------------------------
 # File:         RIFF.pm
 #
-# Description:  Read RIFF/WAV/AVI meta information
+# Description:  Read RIFF/AVI/WAV meta information
 #
 # Revisions:    09/14/2005 - P. Harvey Created
+#               06/28/2017 - PH Added MBWF/RF64 support
 #
 # References:   1) http://www.exif.org/Exif2-2.PDF
 #               2) http://www.vlsi.fi/datasheets/vs1011.pdf
@@ -19,6 +20,7 @@
 #              12) http://abcavi.kibi.ru/infotags.htm
 #              13) http://tech.ebu.ch/docs/tech/tech3285.pdf
 #              14) https://developers.google.com/speed/webp/docs/riff_container
+#              15) https://tech.ebu.ch/docs/tech/tech3306-2009.pdf
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::RIFF;
@@ -27,7 +29,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.38';
+$VERSION = '1.45';
 
 sub ConvertTimecode($);
 
@@ -49,6 +51,30 @@ my %riffMimeType = (
     WV   => 'audio/x-wavpack',
 );
 
+# character sets for recognized Windows code pages
+my %code2charset = (
+    0     => 'Latin',
+    65001 => 'UTF8',
+    1252  => 'Latin',
+    1250  => 'Latin2',
+    1251  => 'Cyrillic',
+    1253  => 'Greek',
+    1254  => 'Turkish',
+    1255  => 'Hebrew',
+    1256  => 'Arabic',
+    1257  => 'Baltic',
+    1258  => 'Vietnam',
+    874   => 'Thai',
+    10000 => 'MacRoman',
+    10029 => 'MacLatin2',
+    10007 => 'MacCyrillic',
+    10006 => 'MacGreek',
+    10081 => 'MacTurkish',
+    10010 => 'MacRomanian',
+    10079 => 'MacIceland',
+    10082 => 'MacCroatian',
+);
+
 %Image::ExifTool::RIFF::audioEncoding = ( #2
     Notes => 'These "TwoCC" audio encoding codes are used in RIFF and ASF files.',
     0x01 => 'Microsoft PCM',
@@ -300,13 +326,13 @@ my %riffMimeType = (
 %Image::ExifTool::RIFF::Main = (
     PROCESS_PROC => \&Image::ExifTool::RIFF::ProcessChunks,
     NOTES => q{
-        The RIFF container format is used various types of fines including WAV, AVI,
+        The RIFF container format is used various types of fines including AVI, WAV,
         WEBP, LA, OFR, PAC and WV.  According to the EXIF specification, Meta
         information is embedded in two types of RIFF C<LIST> chunks: C<INFO> and
         C<exif>, and information about the audio content is stored in the C<fmt >
         chunk.  As well as this information, some video information and proprietary
         manufacturer-specific information is also extracted.
-        
+
         Large AVI videos may be a concatenation of two or more RIFF chunks.  For
         these files, information is extracted from subsequent RIFF chunks as
         sub-documents, but the Duration is calculated for the full video.
@@ -320,6 +346,15 @@ my %riffMimeType = (
         Name => 'BroadcastExtension',
         SubDirectory => { TagTable => 'Image::ExifTool::RIFF::BroadcastExt' },
     },
+    ds64 => { #15
+        Name => 'DataSize64',
+        SubDirectory => { TagTable => 'Image::ExifTool::RIFF::DS64' },
+    },
+    list => 'ListType',  #15
+    labl => { #15
+        Name => 'Label',
+        SubDirectory => { TagTable => 'Image::ExifTool::RIFF::Label' },
+    },
     LIST_INFO => {
         Name => 'Info',
         SubDirectory => { TagTable => 'Image::ExifTool::RIFF::Info' },
@@ -351,6 +386,14 @@ my %riffMimeType = (
             ProcessProc => \&Image::ExifTool::RIFF::ProcessChunks,
         },
     },
+    LIST_pntx => { #Andras Salamon (Q-S1 AVI)
+        Name => 'PentaxData2',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Pentax::AVI',
+            ProcessProc => \&Image::ExifTool::RIFF::ProcessChunks,
+        },
+    },
+    # seen LIST_JUNK
     JUNK => [
         {
             Name => 'OlympusJunk',
@@ -408,6 +451,26 @@ my %riffMimeType = (
         Name => 'Olym',
         SubDirectory => { TagTable => 'Image::ExifTool::Olympus::WAV' },
     },
+    fact => {
+        Name => 'NumberOfSamples',
+        RawConv => 'Get32u(\$val, 0)',
+    },
+   'cue ' => {
+        Name => 'CuePoints',
+        Binary => 1,
+    },
+    afsp => { },
+    IDIT => {
+        Name => 'DateTimeOriginal',
+        Description => 'Date/Time Original',
+        Groups => { 2 => 'Time' },
+        ValueConv => 'Image::ExifTool::RIFF::ConvertRIFFDate($val)',
+        PrintConv => '$self->ConvertDateTime($val)',
+    },
+    CSET => {
+        Name => 'CharacterSet',
+        SubDirectory => { TagTable => 'Image::ExifTool::RIFF::CSET' },
+    },
 #
 # WebP-specific tags
 #
@@ -534,12 +597,50 @@ my %riffMimeType = (
     },
 );
 
+# 64-bit chunk sizes (ref 15)
+%Image::ExifTool::RIFF::DS64 = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 2 => 'Audio' },
+    FORMAT => 'int64u',
+    NOTES => q{
+        64-bit data sizes for MBWF/RF64 files.  See
+        L<https://tech.ebu.ch/docs/tech/tech3306-2009.pdf> for the specification.
+    },
+    0 => {
+        Name => 'RIFFSize64',
+        PrintConv => \&Image::ExifTool::ConvertFileSize,
+    },
+    1 => {
+        Name => 'DataSize64',
+        DataMember => 'DataSize64',
+        RawConv => '$$self{DataSize64} = $val',
+        PrintConv => \&Image::ExifTool::ConvertFileSize,
+    },
+    2 => 'NumberOfSamples64',
+    # (after this comes a table of size overrides for chunk
+    #  types other than 'data', but since these are currently
+    #  very unlikely, support for these is not yet implemented)
+);
+
+# cue point labels (ref 15)
+%Image::ExifTool::RIFF::Label = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 2 => 'Audio' },
+    FORMAT => 'int32u',
+    0 => 'LabelID',
+    1 => {
+        Name => 'LabelText',
+        Format => 'string[$size-4]',
+    },
+);
+
 # Sub chunks of INFO LIST chunk
 %Image::ExifTool::RIFF::Info = (
     PROCESS_PROC => \&Image::ExifTool::RIFF::ProcessChunks,
     GROUPS => { 2 => 'Audio' },
+    FORMAT => 'string',
     NOTES => q{
-        RIFF INFO tags found in WAV audio and AVI video files.  Tags which are part
+        RIFF INFO tags found in AVI video and WAV audio files.  Tags which are part
         of the EXIF 2.3 specification have an underlined Tag Name in the HTML
         version of this documentation.  Other tags are found in AVI files generated
         by some software.
@@ -698,6 +799,15 @@ my %riffMimeType = (
         },
         PrintConv => '$self->ConvertDateTime($val)',
     },
+    # not observed, but apparently part of the standard:
+    IDIT => {
+        Name => 'DateTimeOriginal',
+        Description => 'Date/Time Original',
+        Groups => { 2 => 'Time' },
+        ValueConv => 'Image::ExifTool::RIFF::ConvertRIFFDate($val)',
+        PrintConv => '$self->ConvertDateTime($val)',
+    },
+    ISMP => 'TimeCode',
 );
 
 # Sub chunks of EXIF LIST chunk
@@ -709,11 +819,11 @@ my %riffMimeType = (
     erel => 'RelatedImageFile',
     etim => { Name => 'TimeCreated', Groups => { 2 => 'Time' } },
     ecor => { Name => 'Make',        Groups => { 2 => 'Camera' } },
-    emdl => { Name => 'Model',       Groups => { 2 => 'Camera' } },
+    emdl => { Name => 'Model',       Groups => { 2 => 'Camera' }, Description => 'Camera Model Name' },
     emnt => { Name => 'MakerNotes',  Binary => 1 },
     eucm => {
         Name => 'UserComment',
-        PrintConv => 'Image::ExifTool::Exif::ConvertExifText($self,$val)',
+        PrintConv => 'Image::ExifTool::Exif::ConvertExifText($self,$val,"RIFF:UserComment")',
     },
 );
 
@@ -732,7 +842,7 @@ my %riffMimeType = (
         ValueConv => 'Image::ExifTool::RIFF::ConvertRIFFDate($val)',
         PrintConv => '$self->ConvertDateTime($val)',
     },
-    ISMP => 'Timecode',
+    ISMP => 'TimeCode',
     LIST_strl => {
         Name => 'Stream',
         SubDirectory => { TagTable => 'Image::ExifTool::RIFF::Stream' },
@@ -750,6 +860,20 @@ my %riffMimeType = (
     # (have seen tc_O, tc_A, rn_O and rn_A)
 );
 
+# RIFF character set chunk
+%Image::ExifTool::RIFF::CSET = (
+    PROCESS_PROC => \&Image::ExifTool::RIFF::ProcessBinaryData,
+    GROUPS => { 2 => 'Other' },
+    Format => 'int16u',
+    0 => {
+        Name => 'CodePage',
+        RawConv => '$$self{CodePage} = $val',
+    },
+    1 => 'CountryCode',
+    2 => 'LanguageCode',
+    3 => 'Dialect',
+);
+
 %Image::ExifTool::RIFF::AVIHeader = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
     GROUPS => { 2 => 'Video' },
@@ -1140,6 +1264,9 @@ sub ConvertRIFFDate($)
         # but the Casio QV-3EX writes dates like "2001/ 1/27  1:42PM",
         # and the Casio EX-Z30 writes "2005/11/28/ 09:19"... doh!
         $val = sprintf("%.4d:%.2d:%.2d %.2d:%.2d:00",$1,$2,$3,$4+($6?12:0),$5);
+    } elsif ($val =~ m{(\d{4})[-/](\d+)[-/](\d+)\s+(\d+:\d+:\d+)}) {
+        # the Konica KD500Z writes "2002-12-16  15:35:01\0\0"
+        $val = "$1:$2:$3 $4";
     }
     return $val;
 }
@@ -1266,6 +1393,24 @@ sub ProcessStreamData($$$)
     return 1;
 }
 
+#------------------------------------------------------------------------------
+# Make tag information hash for unknown tag
+# Inputs: 0) Tag table ref, 1) tag ID
+sub MakeTagInfo($$)
+{
+    my ($tagTablePtr, $tag) = @_;
+    my $name = $tag;
+    my $n = ($name =~ s/([\x00-\x1f\x7f-\xff])/'x'.unpack('H*',$1)/eg);
+    # print in hex if tag is numerical
+    $name = sprintf('0x%.4x',unpack('N',$tag)) if $n > 2;
+    AddTagToTable($tagTablePtr, $tag, {
+        Name => "Unknown_$name",
+        Description => "Unknown $name",
+        Unknown => 1,
+        Binary => 1,
+    });
+}
+
 #------------------------------------------------------------------------------
 # Process RIFF chunks
 # Inputs: 0) ExifTool object reference, 1) directory information reference
@@ -1279,10 +1424,20 @@ sub ProcessChunks($$$)
     my $size = $$dirInfo{DirLen};
     my $end = $start + $size;
     my $base = $$dirInfo{Base};
+    my $verbose = $et->Options('Verbose');
+    my $unknown = $et->Options('Unknown');
+    my $charset = $et->Options('CharsetRIFF');
 
-    if ($et->Options('Verbose')) {
-        $et->VerboseDir($$dirInfo{DirName}, 0, $size);
+    unless ($charset) {
+        if ($$et{CodePage}) {
+            $charset = $$et{CodePage};
+        } elsif (defined $charset and $charset eq '0') {
+            $charset = 'Latin';
+        }
     }
+
+    $et->VerboseDir($$dirInfo{DirName}, 0, $size) if $verbose;
+
     while ($start + 8 < $end) {
         my $tag = substr($$dataPt, $start, 4);
         my $len = Get32u($dataPt, $start + 4);
@@ -1313,9 +1468,16 @@ sub ProcessChunks($$$)
                     $start -= $base;
                 }
             } elsif (not $$tagInfo{Binary}) {
-                $val = substr($$dataPt, $start, $len);
-                $val =~ s/\0+$//;   # remove trailing nulls from strings
+                my $format = $$tagInfo{Format} || $$tagTablePtr{FORMAT};
+                if ($format and $format eq 'string') {
+                    $val = substr($$dataPt, $start, $len);
+                    $val =~ s/\0+$//;   # remove trailing nulls from strings
+                    # decode if necessary
+                    $val = $et->Decode($val, $charset) if $charset;
+                }
             }
+        } elsif ($verbose or $unknown) {
+            MakeTagInfo($tagTablePtr, $tag);
         }
         $et->HandleTag($tagTablePtr, $tag, $val,
             DataPt  => $dataPt,
@@ -1339,13 +1501,15 @@ sub ProcessRIFF($$)
 {
     my ($et, $dirInfo) = @_;
     my $raf = $$dirInfo{RAF};
-    my ($buff, $buf2, $type, $mime, $err);
+    my ($buff, $buf2, $type, $mime, $err, $rf64);
     my $verbose = $et->Options('Verbose');
+    my $unknown = $et->Options('Unknown');
 
     # verify this is a valid RIFF file
     return 0 unless $raf->Read($buff, 12) == 12;
-    if ($buff =~ /^RIFF....(.{4})/s) {
-        $type = $riffType{$1};
+    if ($buff =~ /^(RIFF|RF64)....(.{4})/s) {
+        $type = $riffType{$2};
+        $rf64 = 1 if $1 eq 'RF64';
     } else {
         # minimal support for a few obscure lossless audio formats...
         return 0 unless $buff =~ /^(LA0[234]|OFR |LPAC|wvpk)/ and $raf->Read($buf2, 1024);
@@ -1355,6 +1519,7 @@ sub ProcessRIFF($$)
     }
     $mime = $riffMimeType{$type} if $type;
     $et->SetFileType($type, $mime);
+    $$et{VALUE}{FileType} .= ' (RF64)' if $rf64;
     $$et{RIFFStreamType} = '';    # initialize stream type
     SetByteOrder('II');
     my $tagTablePtr = GetTagTable('Image::ExifTool::RIFF::Main');
@@ -1376,8 +1541,10 @@ sub ProcessRIFF($$)
             $pos += 4;
             $tag .= "_$buff";
             $len -= 4;  # already read 4 bytes (the LIST type)
+        } elsif ($tag eq 'data' and $len == 0xffffffff and $$et{DataSize64}) {
+            $len = $$et{DataSize64};
         }
-        $et->VPrint(0, "RIFF '$tag' chunk ($len bytes of data):\n");
+        $et->VPrint(0, "RIFF '${tag}' chunk ($len bytes of data):\n");
         if ($len <= 0) {
             if ($len < 0) {
                 $et->Warn('Invalid chunk length');
@@ -1400,8 +1567,9 @@ sub ProcessRIFF($$)
         }
         # RIFF chunks are padded to an even number of bytes
         my $len2 = $len + ($len & 0x01);
-        if ($$tagTablePtr{$tag} or ($verbose and $tag !~ /^(data|idx1|LIST_movi|RIFF)$/)) {
+        if ($$tagTablePtr{$tag} or (($verbose or $unknown) and $tag !~ /^(data|idx1|LIST_movi|RIFF)$/)) {
             $raf->Read($buff, $len2) == $len2 or $err=1, last;
+            MakeTagInfo($tagTablePtr, $tag) if not $$tagTablePtr{$tag} and ($verbose or $unknown);
             $et->HandleTag($tagTablePtr, $tag, $buff,
                 DataPt  => \$buff,
                 DataPos => 0,   # (relative to Base)
@@ -1415,6 +1583,10 @@ sub ProcessRIFF($$)
             # extract information from remaining file as an embedded file
             $$et{DOC_NUM} = ++$$et{DOC_COUNT}
         } else {
+            if ($len > 0x7fffffff and not $et->Options('LargeFileSupport')) {
+                $et->Warn("Stopped parsing at large $tag chunk (LargeFileSupport not set)");
+                last;
+            }
             $raf->Seek($len2, 1) or $err=1, last;
         }
         $pos += $len2;
@@ -1430,7 +1602,7 @@ __END__
 
 =head1 NAME
 
-Image::ExifTool::RIFF - Read RIFF/WAV/AVI meta information
+Image::ExifTool::RIFF - Read RIFF/AVI/WAV meta information
 
 =head1 SYNOPSIS
 
@@ -1444,7 +1616,7 @@ including AVI videos, WAV audio files and WEBP images.
 
 =head1 AUTHOR
 
-Copyright 2003-2014, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2018, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -1467,6 +1639,8 @@ under the same terms as Perl itself.
 
 =item L<https://developers.google.com/speed/webp/docs/riff_container>
 
+=item L<https://tech.ebu.ch/docs/tech/tech3306-2009.pdf>
+
 =back
 
 =head1 SEE ALSO
