@@ -14,12 +14,12 @@ package Image::ExifTool::Real;
 use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
+use Image::ExifTool::Canon;
 
-$VERSION = '1.00';
+$VERSION = '1.06';
 
 sub ProcessRealMeta($$$);
 sub ProcessRealProperties($$$);
-sub ProcessSerialData($$$);
 
 # Real property types (ref PH)
 my %propertyType = (
@@ -87,15 +87,16 @@ my %metadataFlag = (
 
 %Image::ExifTool::Real::Properties = (
     GROUPS => { 1 => 'Real-PROP', 2 => 'Video' },
-    PROCESS_PROC => \&ProcessSerialData,
+    PROCESS_PROC => \&Image::ExifTool::Canon::ProcessSerialData,
+    VARS => { ID_LABEL => 'Sequence' },
     FORMAT => 'int32u',
-    0  => 'MaxBitRate',
-    1  => 'AvgBitRate',
+    0  => { Name => 'MaxBitrate', PrintConv => 'ConvertBitrate($val)' },
+    1  => { Name => 'AvgBitrate', PrintConv => 'ConvertBitrate($val)' },
     2  => 'MaxPacketSize',
     3  => 'AvgPacketSize',
     4  => 'NumPackets',
-    5 => { Name => 'Duration',      ValueConv => '$val / 1000' },
-    6 => { Name => 'Preroll',       ValueConv => '$val / 1000' },
+    5 => { Name => 'Duration',      ValueConv => '$val / 1000',  PrintConv => 'ConvertDuration($val)' },
+    6 => { Name => 'Preroll',       ValueConv => '$val / 1000',  PrintConv => 'ConvertDuration($val)' },
     7 => { Name => 'IndexOffset',   Unknown => 1 },
     8 => { Name => 'DataOffset',    Unknown => 1 },
     9 => { Name => 'NumStreams',    Format => 'int16u' },
@@ -113,17 +114,18 @@ my %metadataFlag = (
 
 %Image::ExifTool::Real::MediaProps = (
     GROUPS => { 1 => 'Real-MDPR', 2 => 'Video' },
-    PROCESS_PROC => \&ProcessSerialData,
+    PROCESS_PROC => \&Image::ExifTool::Canon::ProcessSerialData,
+    VARS => { ID_LABEL => 'Sequence' },
     FORMAT => 'int32u',
     PRIORITY => 0,  # first stream takes priority
     0  => { Name => 'StreamNumber',  Format => 'int16u' },
-    1  => { Name => 'StreamMaxBitRate' },
-    2  => { Name => 'StreamAvgBitRate' },
+    1  => { Name => 'StreamMaxBitrate', PrintConv => 'ConvertBitrate($val)' },
+    2  => { Name => 'StreamAvgBitrate', PrintConv => 'ConvertBitrate($val)' },
     3  => { Name => 'StreamMaxPacketSize' },
     4  => { Name => 'StreamAvgPacketSize' },
     5  => { Name => 'StreamStartTime' },
-    6  => { Name => 'StreamPreroll',       ValueConv => '$val / 1000' },
-    7  => { Name => 'StreamDuration',      FValueConv => '$val / 1000' },
+    6  => { Name => 'StreamPreroll', ValueConv => '$val / 1000',  PrintConv => 'ConvertDuration($val)' },
+    7  => { Name => 'StreamDuration',ValueConv => '$val / 1000',  PrintConv => 'ConvertDuration($val)' },
     8  => { Name => 'StreamNameLen', Format => 'int8u', Unknown => 1 },
     9  => { Name => 'StreamName',    Format => 'string[$val{8}]' },
     10 => { Name => 'StreamMimeLen', Format => 'int8u', Unknown => 1 },
@@ -233,7 +235,8 @@ my %metadataFlag = (
 
 %Image::ExifTool::Real::ContentDescr = (
     GROUPS => { 1 => 'Real-CONT', 2 => 'Video' },
-    PROCESS_PROC => \&ProcessSerialData,
+    PROCESS_PROC => \&Image::ExifTool::Canon::ProcessSerialData,
+    VARS => { ID_LABEL => 'Sequence' },
     FORMAT => 'int16u',
     0 => { Name => 'TitleLen',      Unknown => 1 },
     1 => { Name => 'Title',         Format => 'string[$val{0}]' },
@@ -266,7 +269,8 @@ my %metadataFlag = (
 
 %Image::ExifTool::Real::AudioV3 = (
     GROUPS => { 1 => 'Real-RA3', 2 => 'Audio' },
-    PROCESS_PROC => \&ProcessSerialData,
+    PROCESS_PROC => \&Image::ExifTool::Canon::ProcessSerialData,
+    VARS => { ID_LABEL => 'Sequence' },
     FORMAT => 'int8u',
     0  => { Name => 'Channels',       Format => 'int16u' },
     1  => { Name => 'Unknown',        Format => 'int16u[3]', Unknown => 1 },
@@ -284,7 +288,8 @@ my %metadataFlag = (
 
 %Image::ExifTool::Real::AudioV4 = (
     GROUPS => { 1 => 'Real-RA4', 2 => 'Audio' },
-    PROCESS_PROC => \&ProcessSerialData,
+    PROCESS_PROC => \&Image::ExifTool::Canon::ProcessSerialData,
+    VARS => { ID_LABEL => 'Sequence' },
     FORMAT => 'int16u',
     0  => { Name => 'FourCC1',        Format => 'undef[4]', Unknown => 1 },
     1  => { Name => 'AudioFileSize',  Format => 'int32u',   Unknown => 1 },
@@ -321,7 +326,8 @@ my %metadataFlag = (
 
 %Image::ExifTool::Real::AudioV5 = (
     GROUPS => { 1 => 'Real-RA5', 2 => 'Audio' },
-    PROCESS_PROC => \&ProcessSerialData,
+    PROCESS_PROC => \&Image::ExifTool::Canon::ProcessSerialData,
+    VARS => { ID_LABEL => 'Sequence' },
     FORMAT => 'int16u',
     0  => { Name => 'FourCC1',        Format => 'undef[4]', Unknown => 1 },
     1  => { Name => 'AudioFileSize',  Format => 'int32u',   Unknown => 1 },
@@ -343,104 +349,19 @@ my %metadataFlag = (
     17 => { Name => 'FourCC3',        Format => 'undef[4]', Unknown => 1 },
 );
 
-#------------------------------------------------------------------------------
-# process a serial stream of binary data
-# Inputs: 0) ExifTool object ref, 1) dirInfo ref, 2) tag table ref
-# Returns: 1 on success
-# Notes: The tagID's for serial stream tags are consecutive indices beginning
-#        at 0, and the corresponding values must be contiguous in memory.
-#        "Unknown" tags must be used to skip padding or unknown values.
-sub ProcessSerialData($$$)
-{
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
-    my $dataPt = $$dirInfo{DataPt};
-    my $offset = $$dirInfo{DirStart};
-    my $size = $$dirInfo{DirLen};
-    my $base = $$dirInfo{Base} || 0;
-    my $verbose = $exifTool->Options('Verbose');
-    my $dataPos = $$dirInfo{DataPos} || 0;
-
-    # temporarily set Unknown option so GetTagInfo() will return unknown tags
-    my $unknown = $exifTool->Options(Unknown => 1);
-
-    $verbose and $exifTool->VerboseDir('SerialData', undef, $size);
-
-    # get default format ('int8u' unless specified)
-    my $defaultFormat = $$tagTablePtr{FORMAT} || 'int8u';
-
-    my ($index, %val);
-    my $pos = 0;
-    for ($index=0; $$tagTablePtr{$index} and $pos <= $size; ++$index) {
-        my $tagInfo = $exifTool->GetTagInfo($tagTablePtr, $index) or last;
-        # don't process tags that fail a condition
-        last unless $tagInfo eq $$tagTablePtr{$index};
-        my $format = $$tagInfo{Format};
-        my $count = 1;
-        if ($format) {
-            if ($format =~ /(.*)\[(.*)\]/) {
-                $format = $1;
-                $count = $2;
-                # evaluate count to allow count to be based on previous values
-                #### eval Format (%val, $size)
-                $count = eval $count;
-                $@ and warn("Format $$tagInfo{Name}: $@"), last;
-            } elsif ($format eq 'string') {
-                # allow string with no specified count to run to end of block
-                $count = ($size > $pos) ? $size - $pos : 0;
-            }
-        } else {
-            $format = $defaultFormat;
-        }
-        my $len = (Image::ExifTool::FormatSize($format) || 1) * $count;
-        last if $pos + $len > $size;
-        my $val = ReadValue($dataPt, $pos+$offset, $format, $count, $size-$pos);
-        last unless defined $val;
-        if ($verbose) {
-            $exifTool->VerboseInfo($index, $tagInfo,
-                Index  => $index,
-                Table  => $tagTablePtr,
-                Value  => $val,
-                DataPt => $dataPt,
-                Size   => $len,
-                Start  => $pos+$offset,
-                Addr   => $pos+$offset+$base+$dataPos,
-                Format => $format,
-                Count  => $count,
-            );
-        }
-        $val{$index} = $val;
-        if ($$tagInfo{SubDirectory}) {
-            my $subTablePtr = GetTagTable($tagInfo->{SubDirectory}->{TagTable});
-            my %dirInfo = (
-                DataPt => \$val,
-                DataPos => $dataPos + $pos,
-                DirStart => 0,
-                DirLen => length($val),
-            );
-            $exifTool->ProcessDirectory(\%dirInfo, $subTablePtr);
-        } elsif (not $$tagInfo{Unknown} or $unknown) {
-            # don't extract zero-length information
-            $exifTool->FoundTag($tagInfo, $val) if $count;
-        }
-        $pos += $len;
-    }
-    $exifTool->Options(Unknown => $unknown);    # restore Unknown option
-    return 1;
-}
-
 #------------------------------------------------------------------------------
 # Process Real NameValueProperties
 # Inputs: 0) ExifTool object reference, 1) dirInfo ref, 2) tag table ref
 # Returns: 1 on success
 sub ProcessRealProperties($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
     my $dirLen = $$dirInfo{DirLen};
     my $pos = $$dirInfo{DirStart};
-    my $verbose = $exifTool->Options('Verbose');
+    my $verbose = $et->Options('Verbose');
 
-    $verbose and $exifTool->VerboseDir('RealProperties', undef, $dirLen);
+    $verbose and $et->VerboseDir('RealProperties', undef, $dirLen);
 
     while ($pos + 6 <= $dirLen) {
 
@@ -469,16 +390,16 @@ sub ProcessRealProperties($$$)
         my $count = int($valLen / Image::ExifTool::FormatSize($format));
         my $val = ReadValue($dataPt, $pos, $format, $count, $dirLen-$pos);
 
-        my $tagInfo = $exifTool->GetTagInfo($tagTablePtr, $tag);
+        my $tagInfo = $et->GetTagInfo($tagTablePtr, $tag);
         unless ($tagInfo) {
             my $tagName;
             ($tagName = $tag) =~ s/\s+//g;
             next unless $tagName =~ /^\w+$/;    # ignore crazy names
             $tagInfo = { Name => ucfirst($tagName) };
-            Image::ExifTool::AddTagToTable($tagTablePtr, $tag, $tagInfo);
+            AddTagToTable($tagTablePtr, $tag, $tagInfo);
         }
         if ($verbose) {
-            $exifTool->VerboseInfo($tag, $tagInfo,
+            $et->VerboseInfo($tag, $tagInfo,
                 Table  => $tagTablePtr,
                 Value  => $val,
                 DataPt => $dataPt,
@@ -489,7 +410,7 @@ sub ProcessRealProperties($$$)
                 Count  => $count,
             );
         }
-        $exifTool->FoundTag($tagInfo, $val);
+        $et->FoundTag($tagInfo, $val);
         $pos += $valLen;
     }
     return 1;
@@ -501,16 +422,16 @@ sub ProcessRealProperties($$$)
 # Returns: 1 on success
 sub ProcessRealMeta($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
     my $dataPos = $$dirInfo{DataPos};
     my $pos = $$dirInfo{DirStart};
     my $dirEnd = $pos + $$dirInfo{DirLen};
-    my $verbose = $exifTool->Options('Verbose');
+    my $verbose = $et->Options('Verbose');
     my $prefix = $$dirInfo{Prefix} || '';
     $prefix and $prefix .= '/';
 
-    $verbose and $exifTool->VerboseDir('RealMetadata', undef, $$dirInfo{DirLen});
+    $verbose and $et->VerboseDir('RealMetadata', undef, $$dirInfo{DirLen});
 
     for (;;) {
         last if $pos + 28 > $dirEnd;
@@ -533,12 +454,12 @@ sub ProcessRealMeta($$$)
         last if $valuePos + $valueLen > $dirEnd;
 
         my $format = $metadataFormat{$type};
-        my $tagInfo = $exifTool->GetTagInfo($tagTablePtr, $tag);
+        my $tagInfo = $et->GetTagInfo($tagTablePtr, $tag);
         unless ($tagInfo) {
             my $tagName = $tag;
             $tagName =~ tr/A-Za-z0-9//dc;
             $tagInfo = { Name => ucfirst($tagName) };
-            Image::ExifTool::AddTagToTable($tagTablePtr, $tag, $tagInfo);
+            AddTagToTable($tagTablePtr, $tag, $tagInfo);
         }
         if ($verbose) {
             $format = 'undef' unless defined $format;
@@ -559,7 +480,7 @@ sub ProcessRealMeta($$$)
             }
             my $count = int($valueLen / Image::ExifTool::FormatSize($format));
             my $val = ReadValue($dataPt, $valuePos, $format, $count, $dirEnd-$valuePos);
-            $exifTool->HandleTag($tagTablePtr, $tag, $val,
+            $et->HandleTag($tagTablePtr, $tag, $val,
                 DataPt => $dataPt,
                 DataPos => $dataPos,
                 Start => $valuePos,
@@ -577,12 +498,12 @@ sub ProcessRealMeta($$$)
                 DirLen => $pos + $size - $dirStart,
                 Prefix => $tag,
             );
-            $exifTool->ProcessDirectory(\%dirInfo, $tagTablePtr);
+            $et->ProcessDirectory(\%dirInfo, $tagTablePtr);
         }
         $pos += $size;  # step to next Metadata structure
     }
     unless ($pos == $dirEnd) {
-        $exifTool->Warn('Format error in Real Metadata');
+        $et->Warn('Format error in Real Metadata');
         return 0;
     }
     return 1;
@@ -594,13 +515,14 @@ sub ProcessRealMeta($$$)
 # Returns: 1 on success, 0 if this wasn't a valid Real file
 sub ProcessReal($$)
 {
-    my ($exifTool, $dirInfo) = @_;
+    my ($et, $dirInfo) = @_;
     my $raf = $$dirInfo{RAF};
     my ($buff, $tag, $vers, $extra, @mimeTypes, %dirCount);
 
     $raf->Read($buff, 8) == 8 or return 0;
     $buff =~ m{^(\.RMF|\.ra\xfd|pnm://|rtsp://|http://)} or return 0;
 
+    my $fast3 = $$et{OPTIONS}{FastScan} && $$et{OPTIONS}{FastScan} == 3;
     my ($type, $tagTablePtr);
     if ($1 eq '.RMF') {
         $tagTablePtr = GetTagTable('Image::ExifTool::Real::Media');
@@ -610,10 +532,10 @@ sub ProcessReal($$)
         $type = 'RA';
     } else {
         $tagTablePtr = GetTagTable('Image::ExifTool::Real::Metafile');
-        my $ext = $exifTool->{FILE_EXT};
+        my $ext = $$et{FILE_EXT};
         $type = ($ext and $ext eq 'RPM') ? 'RPM' : 'RAM';
         require Image::ExifTool::PostScript;
-        my $oldSep = Image::ExifTool::PostScript::SetInputRecordSeparator($raf);
+        local $/ = Image::ExifTool::PostScript::GetInputRecordSeparator($raf) || "\n";
         $raf->Seek(0,0);
         while ($raf->ReadLine($buff)) {
             last if length $buff > 256;
@@ -622,20 +544,21 @@ sub ProcessReal($$)
             if ($type) {
                 # must be a Real file type if protocol is http
                 return 0 if $buff =~ /^http/ and $buff !~ /\.(ra|rm|rv|rmvb|smil)$/i;
-                $exifTool->SetFileType($type);
+                $et->SetFileType($type);
+                return 1 if $fast3;
                 undef $type;
             }
             # save URL or Text from RAM file
             my $tag = $buff =~ m{^[a-z]{3,4}://} ? 'url' : 'txt';
-            $exifTool->HandleTag($tagTablePtr, $tag, $buff);
+            $et->HandleTag($tagTablePtr, $tag, $buff);
         }
-        $/ = $oldSep if $oldSep;
         return 1;
     }
 
-    $exifTool->SetFileType($type);
+    $et->SetFileType($type);
+    return 1 if $fast3;
     SetByteOrder('MM');
-    my $verbose = $exifTool->Options('Verbose');
+    my $verbose = $et->Options('Verbose');
 #
 # Process RealAudio file
 #
@@ -644,12 +567,12 @@ sub ProcessReal($$)
         $tag = ".ra$vers";
         my $fpos = $raf->Tell();
         unless ($raf->Read($buff, 512)) {
-            $exifTool->Warn('Error reading audio header');
+            $et->Warn('Error reading audio header');
             return 1;
         }
-        my $tagInfo = $exifTool->GetTagInfo($tagTablePtr, $tag);
+        my $tagInfo = $et->GetTagInfo($tagTablePtr, $tag);
         if ($verbose > 2) {
-            $exifTool->VerboseInfo($tag, $tagInfo, DataPt => \$buff, DataPos => $fpos);
+            $et->VerboseInfo($tag, $tagInfo, DataPt => \$buff, DataPos => $fpos);
         }
         if ($tagInfo) {
             my $subTablePtr = GetTagTable($tagInfo->{SubDirectory}->{TagTable});
@@ -659,9 +582,9 @@ sub ProcessReal($$)
                 DirLen   => length $buff,
                 DirStart => 0,
             );
-            $exifTool->ProcessDirectory(\%dirInfo, $subTablePtr);
+            $et->ProcessDirectory(\%dirInfo, $subTablePtr);
         } else {
-            $exifTool->Warn('Unsupported RealAudio version');
+            $et->Warn('Unsupported RealAudio version');
         }
         return 1;
     }
@@ -671,7 +594,7 @@ sub ProcessReal($$)
     # skip the rest of the RM header
     my $size = unpack('x4N', $buff);
     unless ($raf->Seek($size - 8, 1)) {
-        $exifTool->Warn('Error seeking in file');
+        $et->Warn('Error seeking in file');
         return 0;
     }
 
@@ -681,23 +604,23 @@ sub ProcessReal($$)
         ($tag, $size, $vers) = unpack('a4Nn', $buff);
         last if $tag eq "\0\0\0\0";
         if ($verbose) {
-            $exifTool->VPrint(0, "$tag chunk ($size bytes):\n");
+            $et->VPrint(0, "$tag chunk ($size bytes):\n");
         } else {
             last if $tag eq 'DATA'; # stop normal parsing at DATA tag
         }
         if ($size & 0x80000000) {
-            $exifTool->Warn('Bad chunk header');
+            $et->Warn('Bad chunk header');
             last;
         }
-        my $tagInfo = $exifTool->GetTagInfo($tagTablePtr, $tag);
+        my $tagInfo = $et->GetTagInfo($tagTablePtr, $tag);
         if ($tagInfo and $$tagInfo{SubDirectory}) {
             my $fpos = $raf->Tell();
             unless ($raf->Read($buff, $size-10) == $size-10) {
-                $exifTool->Warn("Error reading $tag chunk");
+                $et->Warn("Error reading $tag chunk");
                 last;
             }
             if ($verbose > 2) {
-                $exifTool->VerboseInfo($tag, $tagInfo, DataPt => \$buff, DataPos => $fpos);
+                $et->VerboseInfo($tag, $tagInfo, DataPt => \$buff, DataPos => $fpos);
             }
             my $subTablePtr = GetTagTable($tagInfo->{SubDirectory}->{TagTable});
             my %dirInfo = (
@@ -707,30 +630,30 @@ sub ProcessReal($$)
                 DirStart => 0,
             );
             if ($dirCount{$tag}) {
-                $exifTool->{SET_GROUP1} = '+' . ++$dirCount{$tag};
+                $$et{SET_GROUP1} = '+' . ++$dirCount{$tag};
             } else {
                 $dirCount{$tag} = 1;
             }
-            $exifTool->ProcessDirectory(\%dirInfo, $subTablePtr);
-            delete $exifTool->{SET_GROUP1};
+            $et->ProcessDirectory(\%dirInfo, $subTablePtr);
+            delete $$et{SET_GROUP1};
             # keep track of stream MIME types
-            my $mime = $exifTool->{RealStreamMime};
+            my $mime = $$et{RealStreamMime};
             if ($mime) {
-                delete $exifTool->{RealStreamMime};
+                delete $$et{RealStreamMime};
                 $mime =~ s/\0.*//s;
                 push @mimeTypes, $mime unless $mime =~ /^logical-/;
             }
         } else {
             unless ($raf->Seek($size-10, 1)) {
-                $exifTool->Warn('Error seeking in file');
+                $et->Warn('Error seeking in file');
                 last;
             }
         }
     }
     # override MIMEType with stream MIME type if we only have one stream
     if (@mimeTypes == 1 and length $mimeTypes[0]) {
-        $exifTool->{VALUE}->{MIMEType} = $mimeTypes[0];
-        $verbose and $exifTool->VPrint(0, "  MIMEType = $mimeTypes[0]\n");
+        $$et{VALUE}{MIMEType} = $mimeTypes[0];
+        $et->VPrint(0, "  MIMEType = $mimeTypes[0]\n");
     }
 #
 # Process footer containing Real metadata and ID3 information
@@ -748,19 +671,19 @@ sub ProcessReal($$)
                 DirLen => length($buff) - 8,
             );
             my $tagTablePtr = GetTagTable('Image::ExifTool::Real::Metadata');
-            $exifTool->ProcessDirectory(\%dirInfo, $tagTablePtr);
+            $et->ProcessDirectory(\%dirInfo, $tagTablePtr);
         } else {
-            $exifTool->Warn('Bad metadata footer');
+            $et->Warn('Bad metadata footer');
         }
         if ($raf->Seek(-128, 2) and $raf->Read($buff, 128) == 128 and $buff =~ /^TAG/) {
-            $exifTool->VPrint(0, "ID3v1:\n");
+            $et->VPrint(0, "ID3v1:\n");
             my %dirInfo = (
                 DataPt => \$buff,
                 DirStart => 0,
                 DirLen => length($buff),
             );
             my $tagTablePtr = GetTagTable('Image::ExifTool::ID3::v1');
-            $exifTool->ProcessDirectory(\%dirInfo, $tagTablePtr);
+            $et->ProcessDirectory(\%dirInfo, $tagTablePtr);
         }
     }
     return 1;
@@ -792,7 +715,7 @@ little-endian, but the Real format is big-endian.
 
 =head1 AUTHOR
 
-Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2018, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
