@@ -9,7 +9,7 @@
 #               2) http://homepage3.nifty.com/kamisaka/makernote/makernote_ricoh.htm
 #               3) Tim Gray private communication (GR)
 #               4) https://github.com/atotto/ricoh-theta-tools/
-#               5) Iliah Borg private communication (LibRaw)
+#               IB) Iliah Borg private communication (LibRaw)
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::Ricoh;
@@ -19,7 +19,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.31';
+$VERSION = '1.32';
 
 sub ProcessRicohText($$$);
 sub ProcessRicohRMETA($$$);
@@ -143,7 +143,7 @@ my %ricohLensIDs = (
             9 => 'Warm White Fluorescent',
             10 => 'Manual',
             11 => 'Kelvin',
-            12 => 'Shade', #5
+            12 => 'Shade', #IB
         },
     }],
     0x1004 => { #3
@@ -208,7 +208,7 @@ my %ricohLensIDs = (
     0x100b => { #3
         Name => 'FlashExposureComp',
         Writable => 'rational64s',
-        PrintConv => '$val ? sprintf("%+.1f",$val) : $val',
+        PrintConv => '$val ? sprintf("%+.1f",$val) : 0',
         PrintConvInv => '$val',
     },
     0x100c => { #3
@@ -512,6 +512,7 @@ my %ricohLensIDs = (
         OffsetPair => 30,   # associated byte count tagID
         DataTag => 'PreviewImage',
         Protected => 2,
+        WriteGroup => 'MakerNotes',
         # prevent preview from being written to MakerNotes of DNG images
         RawConvInv => q{
             return $val if $$self{FILE_TYPE} eq "JPEG";
@@ -525,6 +526,7 @@ my %ricohLensIDs = (
         OffsetPair => 28,   # point to associated offset
         DataTag => 'PreviewImage',
         Protected => 2,
+        WriteGroup => 'MakerNotes',
         RawConvInv => q{
             return $val if $$self{FILE_TYPE} eq "JPEG";
             warn "\n"; # suppress warning
@@ -1123,7 +1125,7 @@ interpret Ricoh maker notes EXIF meta information.
 
 =head1 AUTHOR
 
-Copyright 2003-2016, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2018, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
