@@ -17,7 +17,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.06';
+$VERSION = '1.08';
 
 sub ProcessComments($$$);
 
@@ -96,6 +96,7 @@ sub ProcessComments($$$);
     COVERARTMIME => { Name => 'CoverArtMIMEType' },
     COVERART  => {
         Name => 'CoverArt',
+        Groups => { 2 => 'Preview' },
         Notes => 'base64-encoded image',
         ValueConv => q{
             require Image::ExifTool::XMP;
@@ -115,6 +116,22 @@ sub ProcessComments($$$);
     PRODUCER    => { Name => 'Producer' },
     COMPOSER    => { Name => 'Composer' },
     ACTOR       => { Name => 'Actor' },
+    # Opus tags
+    ENCODER     => { Name => 'Encoder' },
+    ENCODER_OPTIONS => { Name => 'EncoderOptions' },
+    METADATA_BLOCK_PICTURE => {
+        Name => 'Picture',
+        Binary => 1,
+        # ref https://wiki.xiph.org/VorbisComment#METADATA_BLOCK_PICTURE
+        RawConv => q{
+            require Image::ExifTool::XMP;
+            Image::ExifTool::XMP::DecodeBase64($val);
+        },
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::FLAC::Picture',
+            ByteOrder => 'BigEndian',
+        },
+    },
 );
 
 # Vorbis composite tags
@@ -139,7 +156,7 @@ Image::ExifTool::AddCompositeTags('Image::ExifTool::Vorbis');
 # Returns: 1 on success, otherwise returns 0 and sets a Warning
 sub ProcessComments($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
     my $dataPos = $$dirInfo{DataPos};
     my $pos = $$dirInfo{DirStart} || 0;
@@ -157,7 +174,7 @@ sub ProcessComments($$$)
         my ($tag, $val);
         if (defined $num) {
             $buff =~ /(.*?)=(.*)/s or last;
-            ($tag, $val) = ($1, $2);
+            ($tag, $val) = (uc $1, $2);
             # Vorbis tag ID's are all capitals, so they may conflict with our internal tags
             # --> protect against this by adding a trailing underline if necessary
             $tag .= '_' if $Image::ExifTool::specialTags{$tag};
@@ -165,7 +182,7 @@ sub ProcessComments($$$)
             $tag = 'vendor';
             $val = $buff;
             $num = ($pos + 4 < $end) ? Get32u($dataPt, $pos) : 0;
-            $exifTool->VPrint(0, "  + [Vorbis comments with $num entries]\n");
+            $et->VPrint(0, "  + [Vorbis comments with $num entries]\n");
             $pos += 4;
         }
         # add tag to table unless it exists already
@@ -174,9 +191,10 @@ sub ProcessComments($$$)
             # remove invalid characters in tag name and capitalize following letters
             $name =~ s/[^\w-]+(.?)/\U$1/sg;
             $name =~ s/([a-z0-9])_([a-z])/$1\U$2/g;
+            $et->VPrint(0, "  | [adding $tag]\n");
             AddTagToTable($tagTablePtr, $tag, { Name => $name });
         }
-        $exifTool->HandleTag($tagTablePtr, $tag, $exifTool->Decode($val, 'UTF8'),
+        $et->HandleTag($tagTablePtr, $tag, $et->Decode($val, 'UTF8'),
             Index   => $index,
             DataPt  => $dataPt,
             DataPos => $dataPos,
@@ -187,7 +205,7 @@ sub ProcessComments($$$)
         $num-- or return 1;
         $index = (defined $index) ? $index + 1 : 0;
     }
-    $exifTool->Warn('Format error in Vorbis comments');
+    $et->Warn('Format error in Vorbis comments');
     return 0;
 }
 
@@ -210,7 +228,7 @@ information from Ogg Vorbis audio headers.
 
 =head1 AUTHOR
 
-Copyright 2003-2012, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2018, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
